import pandas as pd

def moving_average(stock_data, size):
    """
    Calculates the moving average of a stock price for a given range of dates

    Parameters
    ----------
    stock_data : string
        A string corresponding to the name of the stock.
        
    size : int
        The size of the period that we're interested in. 

    Returns
    --------
    moving_avg : dataframe
        A dataframe representing the 200 day moving average, for example, of a stock price.

    Examples
    --------
    >>> moving_average("MSFT", 200)
    """
    data = pd.read_csv('../../data/'+stock_data+'.csv')
    data.index = pd.to_datetime(data["Date"], utc=True).dt.date
    mov_avg = {}
    mov_avg[f"{size}MA"] = data["Close"].rolling(window=size).mean()
    moving_avg = pd.DataFrame(mov_avg).reset_index()
    return moving_avg



import matplotlib.pyplot as plt
from stocksignals.calc_moving_avg import moving_average as ma
import pandas as pd

def plot_ma_10_20days(stock_symbol):
    """
    Plot stock price and corresponding 10 and 20 day moving average.
    Parameters
    ----------        
    stock_symbol : string
        Ticker symbol of the stock for which the plot is created
    Returns
    --------
    Matplotlib chart
        A line chart showing price data and corresponding 10 and 20 day 
        moving average line for a stock.
    Examples
    --------
    >>> plot_ma_10_20days("MSFT")
    """
    if not isinstance(stock_symbol, str):
        raise TypeError("Sorry, the input must be a string")
    data = pd.read_csv('../../data/'+stock_symbol+'.csv')
    data["Date"] = pd.to_datetime(data["Date"], utc=True).dt.date
    mov_avg_10 = ma(stock_symbol, 10) 
    mov_avg_20 = ma(stock_symbol, 20)

    plt.figure(figsize=(8,6))
    plt.plot(data.iloc[-200:, 0],  # date
             data.iloc[-200:, 4], "b-", label='Price')  # close
    plt.plot(mov_avg_10.iloc[-200:, 0],
             mov_avg_10.iloc[-200:, 1], "r--", label = '10-days SMA')
    plt.plot(mov_avg_20.iloc[-200:, 0],
             mov_avg_20.iloc[-200:, 1], "g--", label = '20-days SMA')
    plt.xticks(rotation=90)
    plt.xlabel("Date (YY-MM-DD)")
    plt.ylabel("Closing Price (USD)")
    plt.title("10 & 20 day moving average vs closing price")
    plt.legend()
    plt.show()


import matplotlib.pyplot as plt
from stocksignals import bbands as bb
import pandas as pd

def plot_bbands(stock_symbol):
    """
    Plot stock price and corresponding upper and lower Bollinger bands
    for the last 200 trading days.
    
        Parameters
    ----------        
    stock_symbol : string
        Ticker symbol of the stock for which the plot is created
    Returns
    --------
    Matplotlib chart
        A line chart showing price data and corresponding upper
        and lower Bollinger bands.
    Examples
    --------
    >>> plot_bbands("MSFT")
    """
    
    # Check if stock_symbol is a string
    if not isinstance(stock_symbol, str):
        raise TypeError("Sorry, the input must be a string")

        
    # Load the data
    data = pd.read_csv('../../data/'+stock_symbol+'.csv')
    data["Date"] = pd.to_datetime(data["Date"], utc=True).dt.date

    
    # Obtain a DataFrame with the Bollinger bands
    bbands = bb.get_bbands(stock_symbol).reset_index()

    # Create a plot with closing price surrounded by the two bands
    plt.figure(figsize=(8,6))
    plt.plot(data.iloc[-200:, 0],  # date
             data.iloc[-200:, 4], "b-", label='Price')  # close
    plt.plot(bbands.iloc[-200:, 0],
             bbands.iloc[-200:, 1], "r--", label = 'Upper Bollinger band')
    plt.plot(bbands.iloc[-200:, 0],
             bbands.iloc[-200:, 2], "g--", label = 'Lower Bollinger band')
    plt.xticks(rotation=90)
    plt.xlabel("Date (YYYY-MM)")
    plt.ylabel("Price level (USD)")
    plt.title(f"20-day Bollinger bands vs closing price {stock_symbol}")
    plt.legend()
    plt.show()


import pandas as pd
def get_bbands(stock_data, rate = 20):
    """
    Calculates upper and lower Bollinger bands for a stock using
    a day-range; default day range = 20 days
    
    Parameters
    ----------
    data : a data file containing all available stock data
            from Yahoo finance
    rate : int
        number of days, default = 20 days
    
    Returns
    --------
    bbands : DataFrame
        A Pandas DataFrame with upper and lower Bollinger band values
        for the period of availability of the data
    
    Examples
    --------
    >>> get_bbands("MSFT")
    """
    # Read stock data from saved file
    data = pd.read_csv('../../data/'+stock_data+'.csv')

    # Compute prerequisites for the bands
    data.index = pd.to_datetime(data["Date"], utc=True).dt.date
    df = data[['Close']]
    move_average = df.rolling(window=20).mean()
    move_standard_deviation = df.rolling(window=20).std()

    # Create dataframe with the two bands
    bands = move_average + 2 * move_standard_deviation
    bands = bands.rename(columns={"Close": "upper_band"})
    bands['lower_band'] = move_average - 2 * move_standard_deviation
    
    return bands


import matplotlib.pyplot as plt
from stocksignals.calc_moving_avg import moving_average as ma
import pandas as pd

def plot_ma_200days(stock):
    """
    Plot stock price and corresponding 200 day moving average.

    Parameters
    ----------        
    stock : string
        name of the stock for which the plot is created

    Returns
    --------
    Matplotlib chart
        A line chart showing price data and corresponding 200 day 
        moving average line for a stock.

    Examples
    --------
    >>> plot_ma_200days("MSFT")
    """

    data = pd.read_csv('../../data/'+stock+'.csv')
    data["Date"] = pd.to_datetime(data["Date"], utc=True).dt.date
    mov_avg = ma(stock, 200)  # always going to be 200

    plt.figure(figsize=(8,6))
    plt.plot(data.iloc[-200:, 0],  # date
             data.iloc[-200:, 4], "b-", label='Price')  # close
    plt.plot(mov_avg.iloc[-200:, 0],
             mov_avg.iloc[-200:, 1], "r--", label = '200-days SMA')
    plt.xticks(rotation=90)
    plt.xlabel("Date (YY-MM-DD)")
    plt.ylabel("Closing Price (USD)")
    plt.title("200 day moving average vs closing price")
    plt.legend()
    plt.show()




# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['stocksignals']

package_data = \
{'': ['*'], 'stocksignals': ['data/*']}

install_requires = \
['matplotlib>=3.6.3,<4.0.0', 'pandas>=1.5.3,<2.0.0', 'yfinance>=0.2.9,<0.3.0']

setup_kwargs = {
    'name': 'stocksignals',
    'version': '0.0.6',
    'description': 'Calculate buy/sell signal indicators for a stock!',
    'long_description': '![Workflow Badge](https://github.com/UBC-MDS/StockSignals/actions/workflows/ci-cd.yml/badge.svg)\n\n# Stocksignals\n\nCalculate buy/sell signal indicators for a stock!\n\n\n## Authors:\n    - Ruslan Dimitrov\n    - Robin Dhillon\n    - Peng Zhang\n    - Chenyang Wang\n\nThis data science project is created for DSCI 524 (Collaborative Software Development); a course in the Master of Data Science program at the University of British Columbia.\n\n## Introduction\n\nThe goal of this project is to develop a package which can be used as a starting point for identifying stock buy/sell signals. Stock investing is a complex process which requires ongoing efforts and there is no one formula or indicator that works all the time. There exists extensive research on how to identify opportunities and profit from stocks. Methodologies for evaluating financial instruments range widely. An investor can utilize macroeconomic research, fundamental analysis, news, analyst reports, or technical analysis. In all those approaches numerical analysis is the underlying common theme.\n\nIn this project, the team aims to evaluate three key technical indicators that can be used to evaluate where the stock price is relative to its historic performance. These indicators use only the stock\'s historic price and are by no means an exhaustive approach to investing. These indicators are:\n\n- 200-day price moving average\n- 10 vs 20-day price moving average\n- 20-day Bollinger bands\n\nTypically when the market and stocks in particular are trading below 200-day moving average, they are considered in a down trend.  When they trade above the 200-day moving average stocks are considered in an uptrend.  The 10-20 day indicator, indicates short term price trend reversals, and can be utilized to trade stocks on a short term basis.  Finally, the Bollinger bands indicate whether a stock price is above or below two standard deviations from its 20 day average price.  Bbands can be used as indicator for short term overbought/oversold stocks.\n\n<!-- #region -->\n## Package details\nThe package consists of 6 functions:\n- `get_data`: The function downloads all available historic price data for a selected stock and saves it as a `csv` file. It utilizes the `yfinance` python package to automate the process. \n- `moving_average`: The function (inside `calc_moving_average.py` module) calculates a moving average, i.e. the average stock closing price over a specified period, which is passed as argument `size` in the function call. It uses the data saved via `get_data`.\n- `plot_ma_200days`: The function plots the 200-day moving average together with the stock price for a specified period, say the past 200 trading days.  It uses the output from the function `calc_moving_avg` to plot the chart.\n- `plot_ma_10_20`: The function plots the 10 and 20-day moving average together with the stock price for a specified period, say the past 200 trading days. It uses the output from function `move_ave_10_20` to plot the chart.\n- `get_bbands`: The function (inside `bbands.py` module) calculates the 20 day Bollinger bands for the existing period of the data and returns a Pandas DataFrame with the respective upper and lower band. It uses data saved via `get_data`.\n- `plot_bbands`: The function plots upper and lower Bollinger bands together with the stock closing price for over the past 200 days. It uses the output from function `bbands` to plot the chart.\n\n\n## Python ecosystem\n\nThere are multiple packages related to utilizing finance data. For example, past projects in DSCI524 have explored various transformations to help analyzing stocks like this one: https://github.com/UBC-MDS/stockAnalyzer.  Other packages like this one https://github.com/UBC-MDS/Stock-Price-Trend-Volatility-Analysis have explored volatility analysis.  \n\nOur aim is to explore specific technical analysis indicators and streamline the process by providing a hands-on package which can be used in daily stock analysis. Furthermore, this package will help streamline the process in order to help automation of the basics of stock screening.\n\n<!-- #endregion -->\n\n## Installation\n\n```bash\n$ pip install stocksignals\n```\n\n## Usage\n\n```bash\n\nfrom stocksignals.get_data import *\nget_data("MSFT")\n\nfrom stocksignals.calc_moving_avg import *\nmoving_average("MSFT", 10)\n\nfrom stocksignals.bbands import *\nget_bbands("MSFT")\n\nfrom stocksignals.plot_bbands import *\nplot_bbands("MSFT")\n\nfrom stocksignals.plot_ma_10_20 import *\nplot_ma_10_20days("MSFT")\n\nfrom stocksignals.plot_ma_200days import *\nplot_ma_200days("MSFT")\n```\n\n<!-- #region -->\n\n\n## Contributing\n\nInterested in contributing? Check out the [contributing](CONTRIBUTING.md) guidelines. Please note that this project is released with a [Code of Conduct](CONDUCT.md). By contributing to this project, you agree to abide by its terms.\n\n## License\n\n`stocksignals` was created by Ruslan Dimitrov, Robin Dhilon, Peng Zhang, Chenyang Wang. It is licensed under the terms of the MIT license.\n\n## Credits\n\n`stocksignals` was created with [`cookiecutter`](https://cookiecutter.readthedocs.io/en/latest/) and the `py-pkgs-cookiecutter` [template](https://github.com/py-pkgs/py-pkgs-cookiecutter).\n<!-- #endregion -->\n',
    'author': 'Ruslan Dimitrov, Robin Dhilon, Peng Zhang, Chenyang Wang',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/UBC-MDS/StockSignals',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import yfinance as yf
import os

def get_data(stock_ticker):
    """
    Downloads all available historical histocical daily data 
    for stock_ticker from Yahoo finance and stores it as a 
    csv file in data folder. If data folder does not exist
    it is created.
    
    Parameters
    ----------
    stock_ticker : string 
        Ticker of the stock such as 'MSFT'
    
    Returns
    --------
    None
    
    Examples
    --------
    >>> get_data("MSFT")
    """
    
    # Create DataFrame with the stock history data
    
    ticker = yf.Ticker(stock_ticker)
    hist = ticker.history(period="max", interval='1d')
    
    # Define output path for saving the data to a csv file
    out_file = "../../data/"+stock_ticker+".csv"

    # Reset index of dataframe in order to retain dates
    hist.reset_index()
    
    
    # Save the DataFrame to a data directory 
    # and make directory if it doesn't exist
    try:
        hist.to_csv(out_file)
    except:
        os.makedirs(os.path.dirname(out_file))
        hist.to_csv(out_file, index=False)